
import sys
import signal
import re
import fnmatch
import time
import os
import exceptions

from optparse import OptionParser

import yum
import yum.config
import yum.Errors
import yum.packages
from rpmUtils.arch import getArchList
from rpmUtils.miscutils import formatRequire, compareEVR
if yum.__version__<"3.0":
    from yum.logger import Logger, SysLogger, LogContainer

if yum.__version__>"2.9.9":
    import output
    from output import CacheProgressCallback, DepSolveProgressCallBack
elif yum.__version__>"2.4.9":
    import output26 as output
    from output26 import CacheProgressCallback, DepSolveProgressCallBack
else:
    import output24 as output
    from output24 import CacheProgressCallback, DepSolveProgressCallBack

from urlgrabber.progress import TextMeter

import callback
import urlgrabber
import urlgrabber.grabber
import guyumpackage

from guyumpackage import computesize

import rpm

version = "0.0.1"

class GuyumInstall(yum.YumBase, output.YumOutput):
    def __init__(self, pkgops = [], sackops = [], options = None):
        yum.YumBase.__init__(self)
        pkgops = ['queryformat']
        options = {'all': 0, 'file': 0, 'queryformat': None, 'archlist': None, 'grouppkgs': 'default', 'whatrequires': 0, 'querytags': 0, 'conffile': '/etc/yum.conf', 'query': 0, 'repoid': [], 'envra': 0, 'group': 0, 'show_dupes': 0, 'cache': 0, 'nevra': 0, 'source': 0, 'version': 0, 'obsoletes': 0, 'provides': 0, 'location': 0, 'nvr': 0, 'whatobsoletes': 0, 'whatprovides': 0, 'tempcache': 0, 'conflicts': 0, 'info': 0, 'resolve': 0, 'changelog': 0, 'list': 0, 'quiet': 0, 'groupmember': 0, 'alldeps': 0, 'pkgnarrow': 'repos', 'requires': 0, 'whatconflicts': 0}
        self.options = options
        self.pkgops = pkgops
        self.sackops = sackops
        self.pkgs = []        
        self.depspkgs = []
        self.searchlist= ['name']
        self.repolist = []
        self.totalamount = 0
        self.logfileobject = None        


    #
    # returns total installation size
    def gettotalamount(self):
        return computesize(self.totalamount)
        
    #
    # resolve dependencies
    def resolvedependencies(self, pkg):        
        # enable chosen repositories 
        for repo in self.repos.repos.values():
            if repo.id==pkg.repoid:
                self.repos.enableRepo(repo.id)
            else:
                self.repos.disableRepo(repo.id)

        if yum.__version__<="3.1.2":
            self.doRepoSetup(thisrepo=pkg.repoid)
        else:
            repos = yum.YumBase._getRepos(self, thisrepo=pkg.repoid, doSetup=True)            
            # Ugly hack
            self._pkgSack=None

        if yum.__version__>="3.0":
            if yum.__version__<="3.1.2":
                self.doRpmDBSetup()
                self.doUpdateSetup()
                
            else:
                yum.YumBase._getUpdates(self)
                yum.YumBase._getRpmDB(self)

        # setup our depsolve progress callback
        if yum.__version__<"3.0":
            dscb = DepSolveProgressCallBack(self.log, self.errorlog)
        else:
            dscb = DepSolveProgressCallBack()

        self.dsCallback = dscb            

        if yum.__version__<="3.1.2":
            self.doSackSetup(thisrepo=pkg.repoid)
        else:
            #self._pkgSack = None
            yum.YumBase._getSacks(self, thisrepo=pkg.repoid)

        #if yum.__version__<"3.0":
        self.doTsSetup()

        if yum.__version__<'2.6':
            self.tsInfo.addInstall(pkg)                    
        else:
            self.install(po=pkg)

        (rescode, restring)=self.resolveDeps()
        return rescode
    
    #
    # Prepares installation for the given package    
    def prepareinstall(self):

        transactionpkgs = []        
        for txmbr in self.tsInfo.getMembers():

            if txmbr.ts_state in ['i', 'u']:
                depspkg = txmbr.po
                newpkg = guyumpackage.guyumpackage()
                newpkg.remotepackage(depspkg, txmbr.ts_state)

                transactionpkgs.append(newpkg)
                self.totalamount += (int)(newpkg.pkgsize)
        return transactionpkgs
        
    #
    # Install package
    def installpackage(self):
        downloadpkgs = []        
        
        for txmbr in self.tsInfo.getMembers():
            if txmbr.ts_state in ['i', 'u']:
                downloadpkgs.append(txmbr.po)
                
        self.downloadPkgs(downloadpkgs)        
                
        # new callback
        newcallback = callback.RPMInstallCallback(output=1)
        newcallback.tsInfo = self.tsInfo

        # from depsolve
        self.initActionTs()
        self.populateTs(keepold=0)
        self.ts.check()
        self.ts.order()                        
        self.runTransaction(cb=newcallback)
        self.cleartransaction()
        
        #if self.logfileobject!=None:
        #    self.logfileobject.flush()
        
    def cleartransaction(self):
        for txmbr in self.tsInfo:
            self.tsInfo.remove(txmbr.po.pkgtup)
        self.totalamount = 0
        
    def init(self):
        if yum.__version__ > '2.6.9' and yum.__version__ < '3.0':
            self.doStartupConfig()
        self.doConfigSetup()
