
import sys
import signal
import re
import fnmatch
import time
import os
import exceptions

from optparse import OptionParser

import yum
import yum.config
import yum.Errors
import yum.packages
from rpmUtils.arch import getArchList
from rpmUtils.miscutils import formatRequire, compareEVR
if yum.__version__<"3.0":
    from yum.logger import Logger, SysLogger, LogContainer

if yum.__version__>"2.9.9":
    import output
    from output import CacheProgressCallback, DepSolveProgressCallBack
elif yum.__version__>"2.4.9":
    import output26 as output
    from output26 import CacheProgressCallback, DepSolveProgressCallBack
else:
    import output24 as output
    from output24 import CacheProgressCallback, DepSolveProgressCallBack

from urlgrabber.progress import TextMeter

import callback
import urlgrabber
import urlgrabber.grabber
import guyumpackage

import rpm

from guyumpackage import computesize, uniquepackages, filterpackages

version = "0.0.1"

oldrepokey = {'name':'name', 'bandwidth':'bandwidth', 'enablegroups':'enablegroups', 
            'gpgcheck':'gpgcheck', 'includepkgs':'includepkgs', 'keepalive':'keepalive',
            'proxy':'proxy', 'proxy_password':'proxy_password', 'proxy_username':'proxy_username',            
            'exclude': 'excludes', 'retries':'retries', 'throttle':'throttle', 'timeout':'timeout',
            'mirrorlist':'mirrorlistfn', 'cachedir':'cachedir', 'gpgkey':'gpgkey', 'pkgdir':'pkgdir',
            'hdrdir':'hdrdir'}



class GuyumQuery(yum.YumBase, output.YumOutput):
    def __init__(self, pkgops = [], sackops = [], options = None):
        yum.YumBase.__init__(self)
        pkgops = ['queryformat']
        options = {'all': 0, 'file': 0, 'queryformat': None, 'archlist': None, 'grouppkgs': 'default', 'whatrequires': 0, 'querytags': 0, 'conffile': '/etc/yum.conf', 'query': 0, 'repoid': [], 'envra': 0, 'group': 0, 'show_dupes': 0, 'cache': 0, 'nevra': 0, 'source': 0, 'version': 0, 'obsoletes': 0, 'provides': 0, 'location': 0, 'nvr': 0, 'whatobsoletes': 0, 'whatprovides': 0, 'tempcache': 0, 'conflicts': 0, 'info': 0, 'resolve': 0, 'changelog': 0, 'list': 0, 'quiet': 0, 'groupmember': 0, 'alldeps': 0, 'pkgnarrow': 'repos', 'requires': 0, 'whatconflicts': 0}
        self.options = options
        self.pkgops = pkgops
        self.sackops = sackops
        self.pkgs = []        
        self.depspkgs = []
        self.searchlist= ['name']
        self.repolist = []
        self.totalamount = 0
        self.logfileobject = None        


    def setlog(self, filename, debuglevel):
        
        self.logfileobject = file(filename, 'a')
        if yum.__version__<"3.0":
            self.errorlog=Logger(threshold=debuglevel, file_object=self.logfileobject)
            self.log = Logger(threshold=debuglevel, file_object=self.logfileobject)
    
    #
    # Returns package for installation     
    def findpackage(self, packageid):
        for pkg in self.pkgs:
            if pkg.pkgId == packageid:
                return pkg    
                    
    #
    #
    # runs local search in RPM database
    def rpmdbsearch(self, packages):
        retpackages = []
        ts = rpm.TransactionSet()        
        
        for pkgtpl in packages:
            newpackages = ts.dbMatch('name', pkgtpl[0])
            name = pkgtpl[0]

            for newpackage in newpackages:
                cmp = compareEVR((None, pkgtpl[1], pkgtpl[2]), (None, newpackage['version'], newpackage['release']))
                if cmp==-1:
                    retpackages.append((pkgtpl[5], newpackage['version'], newpackage['release'], 'd', pkgtpl[0]))
                if cmp==1:
                    retpackages.append((pkgtpl[5], newpackage['version'], newpackage['release'], 'u', pkgtpl[0]))
                if cmp==0:
                    retpackages.append((pkgtpl[5], newpackage['version'], newpackage['release'], 'o', pkgtpl[0]))
                
        return retpackages


    #
    #
    # Search for a package in a given repository
    def searchpackage(self, package, searchlist, repoid):

        for repo in self.repos.repos.values():            
            if repo.id==repoid:                                
                if repo.enabled==0:                    
                    try:
                        self.repos.enableRepo(repoid)                
                    except:
                        print "enableRepo for %s failured" % repoid
            else:                                
                self.repos.disableRepo(repo.id)

        if yum.__version__<="3.1.2":
            self.doRepoSetup(thisrepo=repoid)
        else:
            if yum.__version__=="3.1.3":
            	repos = yum.YumBase._getRepos(self, thisrepo=repoid)            
            else:
                repos = yum.YumBase._getRepos(self, thisrepo=repoid, doSetup=True)            
            # Ugly hack
            self._pkgSack=None
        
        if yum.__version__>="3.0":            
            if yum.__version__<="3.1.2":
                self.doRpmDBSetup()
                self.doUpdateSetup()
                
            else:
                yum.YumBase._getUpdates(self)
                yum.YumBase._getRpmDB(self)                
        
        # Set logging
#        self.repos.setProgressBar(TextMeter(fo=self.log))
#        self.repos.callback = CacheProgressCallback(self.log, self.errorlog, self.filelog)
        
        # setup our failure report for failover
#        freport = (self.failureReport,(),{})
#        self.repos.setFailureCallback(freport)

        # setup our depsolve progress callback
        if yum.__version__<"3.0":
            dscb = DepSolveProgressCallBack(self.log, self.errorlog)
        else:
            dscb = DepSolveProgressCallBack()
        self.dsCallback = dscb            

        packagestosearch = []
        packagestosearch.append(package)
        packages = []        
        
        if yum.__version__<="3.1.2":
            self.doSackSetup(thisrepo=repoid)
        else:
            #self._pkgSack = None
            yum.YumBase._getSacks(self, thisrepo=repoid)
        
        if yum.__version__<"3.0":
            self.doTsSetup()
        
        pkgs = self.searchPackages(searchlist, packagestosearch)
        localpackages = []
        remotepackages = []

        for pkg in pkgs:            
            appendpackage = 1
            newpkg = guyumpackage.guyumpackage()
            if pkg.repoid=='installed' or pkg.repoid==repoid:
                # divide: remote and locale packages            
                if isinstance(pkg, yum.sqlitesack.YumAvailablePackageSqlite) and pkg.repoid!='installed':
                    newpkg.remotepackage(pkg, 'i')
                    remotepackages.append(newpkg)
                    self.pkgs.append(pkg)                
                else:
                    newpkg.localpackage(pkg, 'o')
                    localpackages.append(newpkg)
                                        
        packages = uniquepackages(localpackages, remotepackages)            
        for localpkg in localpackages:
            packages.append(localpkg)
        for remotepkg in remotepackages:
            packages.append(remotepkg)

        packages = filterpackages(packages)

        #if self.logfileobject!=None:
        #    self.logfileobject.flush()

        return packages

    def addsearchfield(self, fieldname):
        self.searchlist.append(fieldname)
    
    def removesearchfield(self, fieldname):
        self.searchlist.remove(fieldname)

    def amountpackages(self):
        return len(self.pkgs)
        
    def getnthpackage(self, i):
        return self.pkgs[i]
    
    def maprepokeys(self, repository, key):
        retvalue = ""
        if yum.__version__>'2.4.9':
            retvalue = getattr(repository, key)
        else:
            retvalue = getattr(repository, oldrepokey[key])
        if key=='gpgkey':
            retvalue=''.join(retvalue)
        return retvalue
    
    def getrepovalue(self, repoid, key):
        repository = None
        retstring = None
        for repo in self.repos.repos.values():
            if repo.id==repoid:                
                repository = repo
        if repository:                        
            retstring = "%s" % self.maprepokeys(repository, key)
        return retstring

    def getrepos(self):
        retrepos = []
        for repo in self.repos.repos.values():            
            if repo.enabled==1:
                enabledstring = "1"
            else:
                enabledstring = "0"
            retrepos.append({'id':repo.id, 'name':repo.name, 'enabled':enabledstring});
        return retrepos
        
    def init(self):
        if yum.__version__ > '2.6.9' and yum.__version__ < '3.0':
            self.doStartupConfig()
        self.doConfigSetup()
